package com.psmreborn.nopsmdrm;

import android.app.Activity;
import android.app.ActivityManager;
import android.content.Context;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.os.Build;
import android.provider.Settings;
import android.util.Log;

import com.rosstonovsky.abxutils.BinaryXmlPullParser;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;
import org.xmlpull.v1.XmlPullParserFactory;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

import eu.chainfire.libsuperuser.Shell;

public class Helper {

    private static Context ctx;
    public static void setContext(Context context){
        ctx = context.getApplicationContext();
    }
    public static boolean isNoPsmDrmAlreadyInstalled() {
        try {
            if(new File(getPsmAppInfo().nativeLibraryDir, "libdefault_real.so").exists()){
                return true;
            }
        } catch (PackageManager.NameNotFoundException e) {
            return false;
        }
        return false;
    }
    public static boolean isPsmInstalled(){
        try {
            ctx.getPackageManager().getApplicationInfo("com.playstation.psstore", 0);
        } catch (PackageManager.NameNotFoundException e) {
            return false;
        }

        return true;
    }
    private static String parseSsaidText(FileInputStream stream) throws XmlPullParserException, IOException {
        XmlPullParserFactory factory = XmlPullParserFactory.newInstance();
        factory.setNamespaceAware(false);
        XmlPullParser xpp = factory.newPullParser();
        xpp.setInput(stream, "UTF-8");
        int eventType = xpp.getEventType();
        while (eventType != XmlPullParser.END_DOCUMENT) {
            if(eventType == XmlPullParser.START_TAG) {
                if(xpp.getName().equalsIgnoreCase("setting")){
                    String packageName = xpp.getAttributeValue(null,"package");
                    if(packageName != null && packageName.equalsIgnoreCase("com.playstation.psstore")){
                        return xpp.getAttributeValue(null,"value");
                    }
                }

            }
            eventType = xpp.next();
        }
        return null;
    }
    private static String parseSsaidBinary(FileInputStream stream) throws XmlPullParserException, IOException {
        // parse binary xml file ...
        BinaryXmlPullParser xpp = new BinaryXmlPullParser();
        xpp.setInput(stream, "UTF-8");
        int eventType = xpp.getEventType();
        while (eventType != BinaryXmlPullParser.END_DOCUMENT) {
            if(eventType == BinaryXmlPullParser.START_TAG) {
                if(xpp.getName().equalsIgnoreCase("setting")){
                    String packageName = xpp.getAttributeValue("package");
                    if(packageName != null && packageName.equalsIgnoreCase("com.playstation.psstore")){
                        return xpp.getAttributeValue("value");
                    }
                }

            }
            eventType = xpp.next();
        }
        return null;
    }

    public static String getAndroidIdOfPsm() throws Exception {
        if(Build.VERSION.SDK_INT >= 26){
            String[] files = Root.findFiles("/data/system", "settings_ssaid.xml");
            File ssaidCacheFile = new File(ctx.getCacheDir(), "settings_ssaid.xml");

            for(String ssaidFile : files){
                Root.copyChmodAndChown(ssaidFile, ssaidCacheFile.getAbsolutePath(), 777, String.valueOf(getAppInfo(ctx.getPackageName()).uid));
                try{
                    FileInputStream stream = new FileInputStream(ssaidCacheFile);
                    return parseSsaidBinary(stream);
                }
                catch (XmlPullParserException e){
                    FileInputStream stream = new FileInputStream(ssaidCacheFile);
                    return parseSsaidText(stream);
                }

            }
            return null;
        }
        else {
            return Settings.Secure.getString(ctx.getContentResolver(), "android_id");
        }
    }

    public static int getMyUid() {
        try{
            ApplicationInfo psmAppInfo = ctx.getPackageManager().getApplicationInfo(ctx.getPackageName(), 0);
            if(psmAppInfo != null) {
                return psmAppInfo.uid;
            }
        }
        catch (PackageManager.NameNotFoundException e) { };
        return 0;
    }
    public static int getPsmUid() {
        try{
            ApplicationInfo psmAppInfo = ctx.getPackageManager().getApplicationInfo("com.playstation.psstore", 0);
            if(psmAppInfo != null) {
                return psmAppInfo.uid;
            }
        }
        catch (PackageManager.NameNotFoundException e) { };
        return 0;
    }

    public static void killPsm(){
        ActivityManager am = (ActivityManager) ctx.getSystemService(Activity.ACTIVITY_SERVICE);
        am.killBackgroundProcesses("com.playstation.psstore");
    }
    public static PackageInfo getPackageInfo(String pkg) throws PackageManager.NameNotFoundException {
        PackageInfo pkgInfo = ctx.getPackageManager().getPackageInfo(pkg, 0);
        return pkgInfo;
    }
    public static ApplicationInfo getAppInfo(String pkg) throws PackageManager.NameNotFoundException {
        ApplicationInfo appInfo = ctx.getPackageManager().getApplicationInfo(pkg, 0);
        return appInfo;
    }
    public static PackageInfo getPsmAppPkg() throws PackageManager.NameNotFoundException {
        return getPackageInfo("com.playstation.psstore");
    }
    public static ApplicationInfo getPsmAppInfo() throws PackageManager.NameNotFoundException {
        return getAppInfo("com.playstation.psstore");
    }
    public static String getDateTime(){
        String formattedDate = new SimpleDateFormat("MM_dd_yyyy_HH_mm_ss").format(new Date());
        return formattedDate;
    }
}
