package com.psmreborn.nopsmdrm;


import android.Manifest;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.provider.Settings;
import android.support.compat.BuildConfig;
import android.support.v4.app.ActivityCompat;
import android.view.View;
import android.widget.Button;

import com.psmreborn.nopsmdrm.pscertified.PlayStationCertified;
import com.psmreborn.nopsmdrm.pscertified.PsCertificatesInstaller;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import eu.chainfire.libsuperuser.Shell;


public class MainActivity extends Activity {
    public void requestPermissionsIfNeeded(String[] permissions) {
        ArrayList<String> needPerms = new ArrayList<String>();
        for(String perm : permissions){
            if(ActivityCompat.checkSelfPermission(this, perm) != PackageManager.PERMISSION_GRANTED){
                needPerms.add(perm);
            }
        }
        if(!needPerms.isEmpty())
            ActivityCompat.requestPermissions(this, needPerms.toArray(new String[0]), 1);
    }
    public boolean checkHavePermissions(String[] permissions) {
        for(String permission : permissions){
            if (ActivityCompat.checkSelfPermission(this, permission) != PackageManager.PERMISSION_GRANTED) {
                return false;
            }
        }
        return true;
    }
    public void requestPermissions() {
        if(Build.VERSION.SDK_INT >= 23){
            requestPermissionsIfNeeded(new String[] {
                    Manifest.permission.WRITE_EXTERNAL_STORAGE,
                    Manifest.permission.KILL_BACKGROUND_PROCESSES,
                    Manifest.permission.CHANGE_WIFI_STATE,
                    Manifest.permission.ACCESS_WIFI_STATE,
                    Manifest.permission.INTERNET,
                    Manifest.permission.READ_PHONE_STATE,
            });
        }
    }
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        Helper.setContext(this);
        requestPermissions();


        (new Startup(this)).execute();
    }

    @Override
    public void onBackPressed(){
        finish();
        System.exit(0);
        super.onBackPressed();
    }

    private void downloadPsmAlert() {
        new AlertDialog.Builder((Activity)this)
                .setTitle("PSM App not found!")
                .setMessage("Would you like to download and install the PlayStation Mobile app?")
                .setCancelable(false)
                .setPositiveButton("Yes", new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        Downloader.downloadAndInstall(MainActivity.this,
                                "http://psmreborn.com/psm-android/Psm1.7.0.apk");
                    }
                })
                .setNegativeButton("No", null).show();
    }

    private void psCertifiedAlert(){
        new AlertDialog.Builder(this)
                .setTitle("PS Certification Missing")
                .setMessage((Build.VERSION.SDK_INT >= 23) ? "Your device appears to not be \"PlayStation Certified\"\nOn android 6+ this requires installing a Magisk module\nWould you like to install \"MagiskCertify.zip\"" :
                                                            "Your device appears to not be \"PlayStation Certified\"\nDo you want to install them?\n(Warning: modifies /system/)")
                .setCancelable(false)
                .setPositiveButton("Yes", new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        (new PsCertificatesInstaller(MainActivity.this)).execute();
                    }
                })
                .setNegativeButton("No", null).show();
    }

    private boolean checkPermsAlert(){
        if(Build.VERSION.SDK_INT >= 23){
            if (!checkHavePermissions(new String[] {
                    Manifest.permission.WRITE_EXTERNAL_STORAGE,
                    Manifest.permission.KILL_BACKGROUND_PROCESSES,
                    Manifest.permission.CHANGE_WIFI_STATE,
                    Manifest.permission.ACCESS_WIFI_STATE,
                    Manifest.permission.INTERNET,
                    Manifest.permission.READ_PHONE_STATE,
            })) {
                new AlertDialog.Builder(this)
                        .setTitle("Missing permissions")
                        .setMessage("Missing required permissions, please grant all requested permissions\nPHONE is needed only to read IMEI\nSTORAGE is needed for backing up existing PSM data")
                        .setCancelable(false)
                        .setPositiveButton("Ok", null).show();

                return false;
            }
            else{
                return true;
            }
        }
        else{
            return true;
        }
    }
    public void installStart(View view) {;
        if(checkPermsAlert()) {
            if(!PlayStationCertified.isPlaystationCertified(this.getApplicationContext())){
                psCertifiedAlert();
            }
            else if(!Helper.isPsmInstalled()) {
                downloadPsmAlert();
            }
            else {
                new NoPsmDrmInstaller(this).execute();
            }
        }
    }

    public void dumpAllRifs(View view){
        if(checkPermsAlert()){
            String rifOutput = new File(Environment.getExternalStorageDirectory(), "psm").getAbsolutePath();
            new AlertDialog.Builder(this)
                    .setTitle("Backup licenses")
                    .setMessage("This will show a black screen for awhile\nthen a PlayStation Mobile game will start\nOnce the game starts, a backup of all your PSM licenses can be found at\n\""+rifOutput+"\"\nDo you want to do this?")
                    .setCancelable(false)
                    .setPositiveButton("Yes", new DialogInterface.OnClickListener() {
                        @Override
                        public void onClick(DialogInterface dialog, int which) {
                            DumpAllRifs.setDumpAllFlagAndStartPsm(MainActivity.this);
                        }
                    })
                    .setNegativeButton("No", null).show();
        }

    }
    public void givePsmOwnership(View view){
        if(checkPermsAlert()){
            new AlertDialog.Builder(this)
                    .setTitle("Fix permissions")
                    .setMessage("On android 10+, games wont always show up in PSM when placed in \"com.playstation.psstore\" folder due to file permissions.\nWould you like to fix all the file permissions for PSM Games? (can take awhile)")
                    .setCancelable(false)
                    .setPositiveButton("Yes", new DialogInterface.OnClickListener() {
                        @Override
                        public void onClick(DialogInterface dialog, int which) {
                            FixPermissions.changePermissionsToPsm(MainActivity.this);
                        }
                    })
                    .setNegativeButton("No", null).show();
        }
    }


}
